/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0a      Michael K Greene <greenemk@cox.net>
 *                      January 2005
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  logger.exe for Syslogd/2 package. Allows manual sending
 *               syslog messages to the deamon by IPC or UDP.
 *
 ***************************************************************************/

#define SYSLOG_NAMES


#include  <os2>
#include  <fstream>
#include  <iostream>
#include  <cstring>
#include  <cstdlib>
#include  <errno.h>
#include  <ctype.h>
#include  <utils>
#include  "getopt"
#include  "syslog.h"

using std::cout;
using std::endl;
using std::strcat;

static void  usage(void);

struct PROGOPTS {
    int send;
    int facility;
    int priority;
    int logflags;
} progopts;


/*
**  LOGGER -- read and log utility
**
**      This routine reads from an input and arranges to write the
**      result on the system log, along with a useful tag.
*/
int main(int argc, char **argv)
{

    char buf[1024];

    progopts.send     = 0;            // default send BEST
    progopts.facility = LOG_USER;     // default FAC
    progopts.priority = LOG_NOTICE;   // default PRI
    progopts.logflags = 0;            // default NONE


    if(argc == 1)
    {
        usage( );
        exit(0);
    }

    int  ch;

    while ((ch = getopt(argc, argv, "a:f:ip:s")) != -1)
    {
        switch (ch)
        {
            case 'a':  progopts.send     = atoi(optarg);      // 0-best 1-UDP 2-IPC
                       break;
            case 'f':  progopts.facility = (atoi(optarg)<<3); // shift FAC 3bits here
                       break;
            case 'i':  progopts.logflags |= LOG_PID;          // log process id
                       break;
            case 'p':  progopts.priority = atoi(optarg);      // log PRI
                       break;
            case 's':  progopts.logflags |= LOG_PERROR;       // log to standard error
                       break;
            case 'h':
            default:   usage();
        }
    }
    // done with args

    int pri = ( progopts.facility | progopts.priority);

    if(progopts.send < 0 || progopts.send > 2) progopts.send = 0;


    // If sending options, call syslog.c lopenlog( ) to set the options
    // before calling syslog( ).
    if(progopts.logflags || progopts.send)
                         lopenlog(NULL, progopts.logflags, 0, progopts.send);


    // log input line if appropriate, test remaining args
    if (argc > 0)
    {
        for(int i = optind; i < argc; i++)
        {
            strcat(buf, argv[i]);
            strcat(buf, " ");
        }
        lsyslog(progopts.send, pri, "%s", buf);
    }
    else cout << "No message" << endl;

    return(0);
}


void usage(void)
{
    cout << endl;
    cout << "syslogctrl v1.0a for OS/2  " << __DATE__ << endl;
    cout << "Compiled with Watcom 1.3   Michael Greene <greenemk@cox.net>"
         << endl << endl;

    cout << "\nlogger: [-a] [-i] [-s] [-f fac] [-p pri] [ message ... ]\n\n"
         << "        -a  send method 0 - best (default)  1 - IPC  2 - UDP   \n"
         << "        -f  Facility 1 - 23                                    \n"
         << "        -p  Priority 0-7                                       \n"
         << "        -i  LOG_PID                                            \n"
         << "        -s  LOG_PERROR                                         \n"
         << "        message ... Mesaage to send.                           \n"
         << endl;

     exit(1);
}

