/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0       Michael K Greene <greenemk@cox.net>
 *                      January 2005
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  Patch together syslogctrl from cron at.exe source. This
 *               executable is used to send commands to syslogd named pipe.
 *
 ***************************************************************************/

#define  INCL_DOSFILEMGR
#define  INCL_DOSNMPIPES
#define  INCL_WINWINDOWMGR

#define  Message_1     1
#define  Message_2     2
#define  Message_3     3
#define  Message_4     4
#define  Message_5     5

#include  <os2.h>
#include  <iostream>
#include  "getopt.h"
#include  "syslogd.hpp"

using std::cout;
using std::endl;
using std::strcpy;
using std::strcmp;
using std::strncmp;


HPIPE  SyslogPipe;  // handle to pipe start requests are made through

void  usage(void);
void  handleinternalmsg(int msgnum);
int   open_pipe( void );
void  dumpstats( void );
void  dumpconf( void );
void  dumpreap( void );
void  sendtest( void );
void  send_shutdown(void);

static  ULONG bytes_read;
static  ULONG bytes_written;
static  char  buf_data[81];


// main() ***********************************************************
//
int main(int argc, char *argv[])
{

    if (argc == 1) usage( );

    int  ch = 0;

    while ((ch = getopt(argc, argv, "hbst:")) != -1)
    {
        switch (ch)
        {
            case 'h': usage( );
                      break;

            case 'b': sendtest( );
                      exit(0);
                      break;

            case 's': send_shutdown( );
                      exit(0);
                      break;

            case 't': if(stricmp(optarg, "stats") == 0) dumpstats( );
                      else if(stricmp(optarg, "conf") == 0) dumpconf( );
                           else if(stricmp(optarg, "reap") == 0) dumpreap( );
                      exit(0);
                      break;

            default:  handleinternalmsg(Message_1);
                      break;
        }
    }

    handleinternalmsg(Message_1);  // If prg gets here then options were totally crap

    return(0);

}  //$* End of Main() ****


// open_pipe ()
//
//  The DosWaitNPipe was wrong in original - wait up to 5 seconds for busy pipe.
//
int open_pipe(void)
{
    ULONG   ulAction;
    int     statflag   = 0;
    int     error_stat = 0;

    while(DosOpen( PIPE_NAME, (PHFILE)&SyslogPipe,
                   &ulAction, 0L, 0L,
                   SC_PIPE_OPEN , SC_PIPE_MODE, NULL))
    {
        cout << ulAction << endl;
        if (DosWaitNPipe(PIPE_NAME, 1000)) ++error_stat;
        if (error_stat == 10)
        {
            statflag = 1;
            break;
        }
    }

    return(statflag);

}  //$* end of open_pipe


// usage( )
//
//  Display usage to console
//
void usage(void)
{
    cout << endl;
    cout << "syslogctrl v1.0a for OS/2  " << __DATE__ << endl;
    cout << "Compiled with Watcom 1.3   Michael Greene <greenemk@cox.net>" << endl;
    cout << "syslogctrl is a utilitity for use with Syslogd/2." << endl << endl;
    cout << "-t stats | conf" << endl;
    cout << "     stats - return the message receive and processed stats" << endl;
    cout << "     conf  - return the current syslogd message filtering as read" << endl;
    cout << "             from current syslog.conf" << endl;
    cout << "-s   send syslogd shutdown signal" << endl;

    exit( 0 );
}


// handleinternalmsg( )
//
//  Message numbers passed are pulled from stringtable and send to stderr.
//  If DEBUG - FALSE and message number > 199 just return.
//
void handleinternalmsg(int msgnum)
{
    char    StringBuf[81];
    HAB     hab;

    (void)WinLoadString(hab, NULLHANDLE, msgnum, sizeof(StringBuf), StringBuf);
    cout << endl << StringBuf << endl;

    if(msgnum < 4)
    {
        handleinternalmsg(Message_5);
        exit(msgnum);
    }

}  //$* end of handleinternalmsg


// dumpstats ( )
//
//  Open the named pipe and send "stats" command. Received data until
//  receive "DONE".
//
//  Provides a dump of messages syslogd has received and processed.
//
void dumpstats( )
{

    if(open_pipe( )) handleinternalmsg(Message_2);

    strcpy(buf_data,"PIPESTATSEND");

    if (DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written))
    {
        DosClose(SyslogPipe);
        handleinternalmsg(Message_3);
    }

    cout << endl << "Current syslogd message stats:" << endl << endl;

    while(TRUE)
    {

        if(DosRead(SyslogPipe, buf_data, sizeof(buf_data), &bytes_read))
        {
            handleinternalmsg(Message_4);
            break;
        }

        if(strcmp(buf_data, "DONE") == 0) break;
        cout << buf_data << endl;
    }

    DosClose(SyslogPipe);

}  //$* end of dumpstats


void dumpreap( )
{

    if(open_pipe( )) handleinternalmsg(Message_2);

    strcpy(buf_data, "PIPEREAPSEND");

    if (DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written))
    {
        DosClose(SyslogPipe);
        handleinternalmsg(Message_3);
    }

    while(TRUE)
    {
        if(DosRead(SyslogPipe, buf_data, sizeof(buf_data), &bytes_read))
        {
            handleinternalmsg(Message_4);
            break;
        }

//    cout << DosRead(SyslogPipe, buf_data, sizeof(buf_data), &bytes_read);

        if(strncmp(buf_data, "DONE", 4) == 0) break;

        cout << buf_data << endl;
    }

    strcpy(buf_data, "OK");
    DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written);

    DosClose(SyslogPipe);

}  //$* end of dumpconf


// dumpconf ()
//
//  Open the named pipe and send "conf" command. Received data until
//  receive "DONE".
//
//  Provides a formatted dump of configuration syslogd is using.
//
void dumpconf( )
{

    if(open_pipe( )) handleinternalmsg(Message_2);

    strcpy(buf_data, "PIPECONFSEND");

    if (DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written))
    {
        DosClose(SyslogPipe);
        handleinternalmsg(Message_3);
    }

    cout << endl;
    cout << "Current syslog.conf filtering:" << endl;
    cout << "FAC ==>             1 1 1 1 1 1 1 1 1 1 2 2 2 2 2" << endl;
    cout << "0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4" << endl;
    cout << "-------------------------------------------------" << endl;

    while(TRUE)
    {
        if(DosRead(SyslogPipe, buf_data, sizeof(buf_data), &bytes_read))
        {
            handleinternalmsg(Message_4);
            break;
        }

//    cout << DosRead(SyslogPipe, buf_data, sizeof(buf_data), &bytes_read);

        if(strncmp(buf_data, "DONE", 4) == 0) break;

        cout << buf_data << endl;
    }

    strcpy(buf_data, "OK");
    DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written);

    DosClose(SyslogPipe);

}  //$* end of dumpconf


// send_shutdown( )
//
//  Open the named pipe and send shutdown command.
//
void send_shutdown( void )
{
    if(open_pipe()) handleinternalmsg(Message_2);

    strcpy(buf_data,"PIPESHUTDOWN");

    if (DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written))
                                       handleinternalmsg(Message_3);
    DosClose(SyslogPipe);

}  //$* end of send_shutdown


void sendtest( )
{
    if(open_pipe( )) handleinternalmsg(Message_2);

    strcpy(buf_data, "BINGO");

    if (DosWrite(SyslogPipe, buf_data, sizeof(buf_data), &bytes_written))
                                       handleinternalmsg(Message_3);
//    *buf_data = 0;

    DosRead(SyslogPipe, buf_data, sizeof(buf_data), &bytes_read);

    cout << "Buffer " << buf_data << endl;
    DosClose(SyslogPipe);

}

