/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0       Michael K Greene <greenemk@cox.net>
 *                      October 2004
 *
 *                      Based on original BSD Syslogd
 *
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  Main syslogd process. Sets up the program defaults and spawns
 *               threads (IPC, TCPIP, and NAMED PIPE) Reads incomming messages
 *               from the shared buffer and sends to the modified syslogd BSD
 *               module.
 *
 ***************************************************************************/

#define  INCL_DOSPROCESS
#define  INCL_DOSSEMAPHORES
#define  INCL_DOSMEMMGR
#define  INCL_DOSERRORS
#define  INCL_WINWINDOWMGR
#define  INCL_DOSDATETIME
#define  INCL_DOSNMPIPES

#include  <os2.h>
#include  <fstream>
#include  <iostream>
#include  <cstdio>
#include  <cstdlib>
#include  <types>
#include  <string.hpp>
#include  <process>
#include  <cstring>
#include  <cstdarg>
#include  <ctype>
#include  <ctime>
#include  <unistd>
#include  <netdb>
#include  <sys/socket.h>
#include  <netinet/in.h>
#include  <arpa\inet.h>
#include  <signal.h>
#include  "getopt.h"
#include  "syslogmsg.hpp"
#include  "syslogd.hpp"

using std::cout;
using std::cin;
using std::cerr;
using std::endl;
using std::atoi;
using std::ifstream;
using std::ofstream;
using std::ios;
using std::ctime;
using std::time;
using std::time_t;
using std::strftime;
using std::localtime;
using std::printf;
using std::sprintf;
using std::snprintf;
using std::vsnprintf;

PSZ szShareMem = SHMEM_NAME;      // Name of shared memory.
SHAREMEM  *pShareMem;             // will point to start (base) of SMEM

static int  startup( void );      // main syslogd startup function
static void shutdown(int signo);  // main syslogd shutdown function

extern void syslogdipc( PVOID );   // spawn IPC thread
extern void syslogdtcp( PVOID );   // spawn TCP thread
extern void syslogdsnmp( PVOID );  // spawn SNMP thread
extern void syslogdpipe( PVOID );  // spawn PIPE thread
extern void logerror(int sev, const char *, ...);
extern void logmsg(int, char *, int);
extern void printline(char *hname, char *msg);
extern void domark(int mark);
extern void init( void );
extern void poplog( void );

void handleinternalmsg(int msgnum, int sev);
char *gettimestamp( void );                            // get timestap IAW RFC 3164

static void usage(void);
static void initialize( void );


// ********** Semaphores **********

// MUX to signal buffer access
PSZ    MEMMUX      = MUX_NAME;
HMTX   mtxMEMMUX   = 0;
BOOL32 MUXSTATE    = FALSE;

// SEM to signal IPC start / stop
PSZ    IPCSEM1    = "\\SEM32\\SYSLOGD\\IPCSEM1";
HEV    hevIPCSEM1 = 0;
ULONG  IPCSEM1Ct  = 0;

// SEM to signal IPC start / stop
PSZ    IPCSEM2    = "\\SEM32\\SYSLOGD\\IPCSEM2";
HEV    hevIPCSEM2 = 0;
ULONG  IPCSEM2Ct  = 0;

// SEM to signal TCP start / stop
PSZ    TCPSEM1    = "\\SEM32\\SYSLOGD\\TCPSEM1";
HEV    hevTCPSEM1 = 0;
ULONG  TCPSEM1Ct  = 0;

// SEM to signal TCP start / stop
PSZ    TCPSEM2    = "\\SEM32\\SYSLOGD\\TCPSEM2";
HEV    hevTCPSEM2 = 0;
ULONG  TCPSEM2Ct  = 0;

// SEM to signal SNMP start / stop
PSZ    SNMPSEM1    = "\\SEM32\\SYSLOGD\\SNMPSEM1";
HEV    hevSNMPSEM1 = 0;
ULONG  SNMPSEM1Ct  = 0;

// SEM to signal SNMP start / stop
PSZ    SNMPSEM2    = "\\SEM32\\SYSLOGD\\SNMPSEM2";
HEV    hevSNMPSEM2 = 0;
ULONG  SNMPSEM2Ct  = 0;

// SEM to sign SYSLOGPIPE start / stop
PSZ    PIPESEM1    = "\\SEM32\\SYSLOGD\\PIPESEM1";
HEV    hevPIPESEM1 = 0;
ULONG  PIPESEM1Ct  = 0;

// SEM to signal SYSLOGPIPE start / stop
PSZ    PIPESEM2    = "\\SEM32\\SYSLOGD\\PIPESEM2";
HEV    hevPIPESEM2 = 0;
ULONG  PIPESEM2Ct  = 0;

// SEM to signal CMDSHUTDOWN
PSZ    PIPESEM3    = "\\SEM32\\SYSLOGD\\PIPESEM3";
HEV    hevPIPESEM3 = 0;
ULONG  PIPESEM3Ct  = 0;

// SEM to signal MARK timer
static  PSZ    MARKSEM     = "\\SEM32\\SYSLDMP\\MARKTIMER";
static  HEV    hevMARKSEM  = 0;
static  ULONG  MARKSEMCt   = 0;
static  HTIMER hMARKEvent  = 0;

// SEM to signal FLUSH timer
static  PSZ    FLUSHSEM     = "\\SEM32\\SYSLDMP\\FLUSHTIMER";
static  HEV    hevFLUSHSEM  = 0;
static  ULONG  FLUSHSEMCt   = 0;
static  HTIMER hFLUSHEvent  = 0;


// ********** Program Structures **********
struct  STATS       stats;       // struct holds messages received/dropped
struct  PROGRAMOPTS prgoptions;  // struct holds program options
extern  HPIPE       SyslogPipe;

static  int  UseNameService;            // make domain name queries

extern  struct CURRENTMSG currentmsg;

char  LocalHostName[MAXHOSTNAMELEN+1]; // our hostname
char  timebuffer[15];


// main( ) Start ****************************************************
//
//
int main(int argc,char *argv[])
{
    char   data[81] = {0};

    // intialize the program options and parse the syslog.conf file
    initialize( );

    // Get program arguments using getopt()
    // Any command line options will override defaults and options found in
    // syslog.conf
    int ch;

    while ((ch = getopt(argc, argv, "df:hkm:nprstxz:")) != -1)
    {
        switch (ch)
        {
            case 'd':  prgoptions.DEBUG++;
                       break;

            case 'f':  prgoptions.cfgfilefull = optarg;
                       break;

            case 'h':  usage( );
                       break;

            case 'k':  prgoptions.klog++;
                       break;

            case 'm':  prgoptions.marktimer = (ULONG)atoi(optarg);  // mark interval
                       break;

            case 'n':  UseNameService = 0;       // turn off DNS queries
                       break;

            case 'p':  prgoptions.logpriority++; // add msg priority to msg logging
                       break;

            case 'r':  prgoptions.norepeat++;    // disable "repeated" compression
                       break;

            case 's':  prgoptions.securemode++;  // no network listen mode */
                       break;

            case 't':  prgoptions.forwardmode++;
                       break;

            case 'x':  prgoptions.threadsnmp = TRUE;
                       break;

            case 'z':  strcpy(prgoptions.altbindaddr, optarg);  // specific NIC to bind
                       break;

            default:   usage( );
                       break;
        }

    }
    if ((argc -= optind) != 0) usage();

    // done with args

    // allocate shared memory and start IPC and TCP thread
    (void)startup( );

    (void)signal(SIGTERM, shutdown);
    (void)signal(SIGINT,  shutdown);

    for(int x = 0; x < 1024; x++) currentmsg.orgmsg[x] = 0;
    currentmsg.orglen = 0;

    // setup mark timer if prgoptions.marktimer > 0 (0 diabled)
    // default 20 minutes
    if(prgoptions.marktimer)
    {
        if(prgoptions.DEBUG) if(prgoptions.DEBUG) handleinternalmsg(DEBUGMSG_4, LOG_DEBUG);
        DosStartTimer(((ULONG)prgoptions.marktimer * 60L * 1000L),
                      (HSEM)hevMARKSEM, &hMARKEvent);
    }

    if(prgoptions.DEBUG) handleinternalmsg(DEBUGMSG_6, LOG_DEBUG);
    DosStartTimer((30L * 60L * 1000L), (HSEM)hevFLUSHSEM, &hFLUSHEvent);

    prgoptions.initialized = 1;

    // send first message, internal that syslogd restarted
    logmsg(LOG_SYSLOG|LOG_INFO, "syslogd: restart", ADDDATE | ADDHOST);

    // if klog option then process the system log files and log them
    if(prgoptions.klog) poplog( );

// cout << "XX" << prgoptions.altbindaddr << endl;

    // working loop *************************************************
    // syslogd execute from here to end until exit
    while(1)
    {
        DosSleep(500);

        // set MEMMUX and wait turn to access the buffer
        if(DosRequestMutexSem(mtxMEMMUX, 60000L) != 0)
                          handleinternalmsg(RUNNINGERR_1, LOG_ERR);
        else
        {
            // scan the buffer for messages
            for(int i = 0; i < (BUFFER_SLOTS); i++)
            {
                // if slot marked as full ==> process
                if(pShareMem->slotstat[i])
                {
                    stats.sysdmprecvmsg++;  // number of msgs received
                    printline(LocalHostName, pShareMem->data[i]);
                    pShareMem->slotstat[i] = 0;
                    for(int x = 0; x < 1024; x++) pShareMem->data[i][x] = 0;
                }
            }
            // release the MUX
            DosReleaseMutexSem(mtxMEMMUX);
        }


        // Query for command shutdown
        DosQueryEventSem(hevPIPESEM2, &PIPESEM2Ct);
        if(PIPESEM2Ct) shutdown( 0 );


        // Query test signal
        DosQueryEventSem(hevPIPESEM3, &PIPESEM3Ct);
        if(PIPESEM3Ct)
        {
            DosPostEventSem(hevPIPESEM2);
            cout << "About to close" << endl;
            // close_pipe( );

        }


        // Query TCP thread for blocked
        DosQueryEventSem(hevTCPSEM1, &TCPSEM1Ct);
        if(TCPSEM1Ct)
        {
            cout << "TCP THREAD BLOCKED" << endl;
            DosResetEventSem(hevTCPSEM1, &TCPSEM1Ct);
        }  //$* endof Query TCP thread


        // Replaces the unix mark timer
        // if -m is 0 disable MARK
        // check mark timer, if time then domark and restart timer
        if(prgoptions.marktimer)
        {
            DosQueryEventSem(hevMARKSEM, &MARKSEMCt);
            if(MARKSEMCt)
            {
                DosStopTimer(hMARKEvent);
                domark( 1 );
                DosResetEventSem(hevMARKSEM, &MARKSEMCt);
                DosStartTimer(((ULONG)prgoptions.marktimer * 60L * 1000L),
                              (HSEM)hevMARKSEM, &hMARKEvent);
            }

        }  //$* end of check mark timer

        // 30 minute timer to flush and check popuplog.os2
        DosQueryEventSem(hevFLUSHSEM, &FLUSHSEMCt);
        if(FLUSHSEMCt)
        {
            if(prgoptions.DEBUG) handleinternalmsg(DEBUGMSG_7, LOG_DEBUG);
            DosStopTimer(hFLUSHEvent);

            // flush without MARK
            domark( 0 );

            // if klog option then process the system log files and log them
            if(prgoptions.klog) poplog( );

            DosResetEventSem(hevFLUSHSEM, &FLUSHSEMCt);
            DosStartTimer((30L * 60L * 1000L), (HSEM)hevFLUSHSEM, &hFLUSHEvent);

        }  //$* end of flush timer


    }  //$* end of working loop

}  //$* end of main


// usage( )
//
//  Display usage to console
//
void usage(void)
{
    cout <<

    "Syslogd/2 v1.0a for OS/2  " << __DATE__ << " Compiled with Watcom 1.3"                << endl <<
    "              Michael Greene <greenemk@cox.net>"                                     << endl << endl <<
    "usage: syslogd [-d][-h][-k][-m mark_interval][-p][-r][-s][-t][[-v][-w][-x]]"         << endl << endl <<
    "   -d    turn on debug information."                                                 << endl << endl <<
    "   -k    klog - process popuplog.os2 with FAC 0 and SEV 2. Initial check of"         << endl <<
    "         popuplog.os2 at syslogd start and check for new entries every 30 minutes"   << endl << endl <<
    "   -m #  mark interval, where X is the interval (in minutes) a mark event is"        << endl <<
    "         written to log files [disabled by default]"                                 << endl << endl <<
    "   -p    priority will be logged in file and console"                                << endl << endl <<
    "   -r    disable message repeating "                                                 << endl << endl <<
    "   -s    enable listen on port 514 for UDP messages "                                << endl << endl <<
    "   -t    enable message forwarding as configured in syslog.conf"                     << endl << endl <<
    "   -x    enable listen on ports 161 and 162 for SNMP messages"                       << endl << endl <<
    endl;

    exit(1);

}  //$* end of usage


// handleinternalmsg( )
//
//  Message numbers passed are pulled from stringtable and send to stderr.
//  If DEBUG - FALSE and message number > 199 just return.
//
void handleinternalmsg(int msgnum, int sev)
{
    if(!prgoptions.DEBUG && msgnum > 199) return;

    char    StringBuf[256];
    HAB     hab;

    (void)WinLoadString(hab, NULLHANDLE, msgnum, sizeof(StringBuf), StringBuf);

    logerror(sev, StringBuf); // toss my error message off to the BSD module

}  //$* end of handleinternalmsg


// startup()
//
//  Handles syslogd startup. Allocate shared memory, clear it, and setup
//  MUXSEM to control access to the memory buffer.  Setup the named pipe.
//  Start the threads based on value of threadicp and threadtcp.
//
int startup( void )
{
    prgoptions.initialized = 0;

    if(prgoptions.DEBUG) handleinternalmsg(DEBUGMSG_3, LOG_DEBUG);

    // get local machine hostname
    (void)gethostname(LocalHostName, sizeof(LocalHostName));
    LocalHostName[sizeof(LocalHostName) - 1] = '\0';
    if(prgoptions.DEBUG) logerror(LOG_DEBUG, "SYSLOGDMP: Hostname: %s", LocalHostName);

    // allocate shared memory for passing messages from IPC and TCP messages
    //
    // pShareMem -pointer to start of allocated memory
    // SHMEM_NAME - shared mem name and SMEM_SIZE - size to allocate
    int flags =  (PAG_COMMIT | PAG_READ | PAG_WRITE);
    if(DosAllocSharedMem((PPVOID)&pShareMem, SHMEM_NAME, SMEM_SIZE, flags) != 0)
    {
        handleinternalmsg(STARTERROR_3, LOG_ERR);
        return(STARTERROR_3);
    }
    if(prgoptions.DEBUG) handleinternalmsg(DEBUGMSG_5, LOG_DEBUG);
    memset(pShareMem, '\0', SMEM_SIZE);  // clear memory chunk
    // create the muxsem
    if(DosCreateMutexSem(MEMMUX, &mtxMEMMUX, DC_SEM_SHARED, MUXSTATE))
    {
        handleinternalmsg(STARTERROR_10, LOG_ERR);
        return(STARTERROR_10);
    }

    // MARK timer - will output a MARK to configured logs in BSD like
    // manner
    if(DosCreateEventSem(MARKSEM, &hevMARKSEM, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_13, LOG_ERR);
        return(STARTERROR_13);
    }

    // FLUSH timer
    if(DosCreateEventSem(FLUSHSEM, &hevFLUSHSEM, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_14, LOG_ERR);
        return(STARTERROR_14);
    }


    // ***** IPC Thread - setup and start *****
    //
    // Function to setup and start ICP thread.  Create SEM1/SEM2,
    // start thread, and wait for thread to set SEM1 signaling thread
    // running. SEM1 reset on exit/return
    if(DosCreateEventSem(IPCSEM1, &hevIPCSEM1, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_4, LOG_ERR);
        return(STARTERROR_5);
    }
    if(DosCreateEventSem(IPCSEM2, &hevIPCSEM2, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_5, LOG_ERR);
        return(STARTERROR_5);
    }

    // start the IPC thread
    int ipcthread = _beginthread(syslogdipc, NULL, (20*1024), NULL);

    // wait for IPC thread signal that it is started, reset and update IPCSEM1Ct
    if(DosWaitEventSem(hevIPCSEM1, 5000)) handleinternalmsg(STARTERROR_6, LOG_ERR);
    else handleinternalmsg(STARTMSG_1, LOG_DEBUG);
    DosResetEventSem(hevIPCSEM1, &IPCSEM1Ct);


    // ***** UDP Thread - setup and start *****
    //
    // Function to setup and start TCP thread.  Create SEM1/SEM2,
    // start thread, and wait for thread to set SEM1 signaling thread
    // running. SEM1 reset on exit/return
    if(DosCreateEventSem(TCPSEM1, &hevTCPSEM1, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_7, LOG_ERR);
        return(STARTERROR_7);
    }

    if(DosCreateEventSem(TCPSEM2, &hevTCPSEM2, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_8, LOG_ERR);
        return(STARTERROR_8);
    }

    // start the TCP thread
    int tcpthread = _beginthread(syslogdtcp, NULL, (20*1024), NULL);

    // wait for TCP thread signal that it is started, reset and update IPCSEM1Ct
    if(DosWaitEventSem(hevTCPSEM1, 5000)) handleinternalmsg(STARTERROR_9, LOG_ERR);
    else if(prgoptions.DEBUG) handleinternalmsg(STARTMSG_2, LOG_DEBUG);

    DosResetEventSem(hevTCPSEM1, &TCPSEM1Ct);


    // ***** SNMP Thread - setup and start *****
    //
    // Function to setup and start SNMP thread.  Create SEM1/SEM2,
    // start thread, and wait for thread to set SEM1 signaling thread
    // running. SEM1 reset on exit/return
    if(prgoptions.threadsnmp)
    {
        if(DosCreateEventSem(SNMPSEM1, &hevSNMPSEM1, DC_SEM_SHARED, FALSE))
        {
            handleinternalmsg(STARTERROR_9, LOG_ERR);
            return(STARTERROR_9);
        }

        if(DosCreateEventSem(SNMPSEM2, &hevSNMPSEM2, DC_SEM_SHARED, FALSE))
        {
            handleinternalmsg(STARTERROR_10, LOG_ERR);
            return(STARTERROR_10);
        }

        // start the SNMP thread
        int snmpthread = _beginthread(syslogdsnmp, NULL, (20*1024), NULL);

        // wait for SNMP thread signal that it is started, reset and update IPCSEM1Ct
        if(DosWaitEventSem(hevSNMPSEM1, 5000)) handleinternalmsg(STARTERROR_9, LOG_ERR);
        else if(prgoptions.DEBUG) handleinternalmsg(STARTMSG_2, LOG_DEBUG);

        DosResetEventSem(hevSNMPSEM1, &SNMPSEM1Ct);
    }


    // ***** PIPE Thread - setup and start *****
    //
    // Function to setup and start PIPE thread.  Create SEM1/SEM2,
    // start thread, and wait for thread to set SEM1 signaling thread
    // running. SEM1 reset on exit/return
    if(DosCreateEventSem(PIPESEM1, &hevPIPESEM1, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_15, LOG_ERR);
        return(STARTERROR_15);
    }

    if(DosCreateEventSem(PIPESEM2, &hevPIPESEM2, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_16, LOG_ERR);
        return(STARTERROR_16);
    }

    if(DosCreateEventSem(PIPESEM3, &hevPIPESEM3, DC_SEM_SHARED, FALSE))
    {
        handleinternalmsg(STARTERROR_16, LOG_ERR);
        return(STARTERROR_16);
    }

    // start the PIPE thread
    int pipethread = _beginthread(syslogdpipe, NULL, (20*1024), NULL);

    // wait for PIPE thread signal that it is started, reset and update IPCSEM1Ct
    if(DosWaitEventSem(hevPIPESEM1, 5000)) handleinternalmsg(STARTERROR_17, LOG_ERR);
    else if(prgoptions.DEBUG) handleinternalmsg(STARTMSG_3, LOG_DEBUG);

    DosResetEventSem(hevPIPESEM1, &PIPESEM1Ct);

    return(0);

}  //$* end of startup


// shutdown( )
//
//  Full shutdown threads, named pipe, kill all the SEMs, and
//  free shared memory.
//
void shutdown(int signo)
{
    // send shutdown message, internal that syslogd stopping
    logmsg(LOG_SYSLOG|LOG_INFO, "syslogd: stopping", ADDDATE | ADDHOST);

    // Received ctrl+c so signal pipe thread shutdown
    if(signo == 4)
    {
        HPIPE  SDSyslogPipe;
        ULONG  ulAction;
        ULONG  bytes_written;
        char   buf_data[ ] = "PIPESHUTDOWN";

        DosOpen(PIPE_NAME, (PHFILE)&SDSyslogPipe, &ulAction, 0L, 0L,
                            SC_PIPE_OPEN , SC_PIPE_MODE, NULL);
        DosWrite(SDSyslogPipe, buf_data, sizeof(buf_data), &bytes_written);
        DosClose(SDSyslogPipe);
    }

    prgoptions.initialized = 0;

    // post IPCSEM2 IPC thread shutdown
    DosPostEventSem(hevIPCSEM2);
    if(DosWaitEventSem(hevIPCSEM1, 10000)) handleinternalmsg(SHUTDOWNMSG_1, LOG_ERR);
    else if(prgoptions.DEBUG) handleinternalmsg(SHUTDOWNMSG_2, LOG_DEBUG);

    // post IPCSEM2 UDP thread shutdown
    DosPostEventSem(hevTCPSEM2);
    if(DosWaitEventSem(hevTCPSEM1, 10000)) handleinternalmsg(SHUTDOWNMSG_3, LOG_ERR);
    else if(prgoptions.DEBUG) handleinternalmsg(SHUTDOWNMSG_4, LOG_DEBUG);

    // post IPCSEM2 UDP thread shutdown
    if(prgoptions.threadsnmp)
    {
        DosPostEventSem(hevSNMPSEM2);
        if(DosWaitEventSem(hevSNMPSEM1, 10000)) handleinternalmsg(SHUTDOWNMSG_3, LOG_ERR);
        else if(prgoptions.DEBUG) handleinternalmsg(SHUTDOWNMSG_4, LOG_DEBUG);
    }

    // check Pipe shutdown
    if(DosWaitEventSem(hevPIPESEM1, 10000)) handleinternalmsg(SHUTDOWNMSG_5, LOG_ERR);
    else if(prgoptions.DEBUG) handleinternalmsg(SHUTDOWNMSG_6, LOG_DEBUG);

    // kill the SEMs
    DosCloseEventSem(hevIPCSEM1);
    DosCloseEventSem(hevIPCSEM2);
    DosCloseEventSem(hevTCPSEM1);
    DosCloseEventSem(hevTCPSEM2);
    DosCloseEventSem(hevSNMPSEM1);
    DosCloseEventSem(hevSNMPSEM2);
    DosCloseEventSem(hevPIPESEM1);
    DosCloseEventSem(hevPIPESEM2);
    DosCloseEventSem(hevPIPESEM3);
    DosCloseMutexSem(mtxMEMMUX);

    // free the shared memory
    DosFreeMem(pShareMem);

    exit(signo);

}  //$* end of shutdown


// initalize(  )
//
//  Setup the default options and call the BSD code to process the syslog.conf
//  file.  Any program options found in syslog.conf will override the defaults.
//
void initialize( void )
{
    if(prgoptions.DEBUG) handleinternalmsg(DEBUGMSG_1, LOG_DEBUG);
    // setup default options
    prgoptions.DEBUG               = 0;
    prgoptions.marktimer           = 0;
    prgoptions.logpriority         = 0;
    prgoptions.securemode          = 0;
    prgoptions.forwardmode         = 0;
    prgoptions.norepeat            = 0;
    prgoptions.initialized         = 0;
    prgoptions.cfgfilename         = SYSLOGDCONF;
    prgoptions.cfgfilefull         = getos2path("ETC", prgoptions.cfgfilename);
//    prgoptions.altbindaddr         = ;
    prgoptions.klog                = 0;
    *prgoptions.popupfile          = 0;
    prgoptions.threadipc           = TRUE;  // flag to indicate IPC thread start/running
    prgoptions.threadtcp           = FALSE; // flag to indicate TCP thread start/running
    prgoptions.threadsnmp          = FALSE;

    // call to the BSD function
    init( );

}  //$* end of initialize


// gettimestamp( )
//
//  return the date / time RFC3164 format
//
char *gettimestamp( void )
{
    // TIMESTAMP format "Mmm dd hh:mm:ss"
    time_t TimeOfDay;
    char *p;

    TimeOfDay = time( NULL );

    strftime(timebuffer, 15, "%b %d %T", localtime(&TimeOfDay));  // timestamp
    p = timebuffer;

    return(p);

}  //$* end  of gettimestamp



