/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0       Michael K Greene <greenemk@cox.net>
 *                      September 2004
 *
 *                      Based on original BSD Syslogd
 *
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  Syslogd/2 header file various items defined.
 *
 ***************************************************************************/

#ifndef _SYSLOGD_HPP
#define _SYSLOGD_HPP

#define  SYSLOGDCONF       "syslog.conf"

#define UT_NAMESIZE     32
#define MAXLINE         1024            /* maximum line length */
#define LINE_MAX        MAXLINE
#define MAXSVLINE       120             /* maximum saved line length */
#define TIMERINTVL      30              /* interval for checking flush, mark */

#define _PATH_UNIX "os2kernel"
#define _POSIX_PATH_MAX _MAX_PATH

#define DEFUPRI         (LOG_USER|LOG_NOTICE)
#define DEFSPRI         (LOG_KERN|LOG_CRIT)
#define FDMASK(fd)      (1 << (fd))

#define dprintf         if (Debug) printf

#define  PIPE_NAME       "\\PIPE\\SYSLOGD"
#define  PIPE_OPEN_MODE  NP_ACCESS_DUPLEX | NP_NOINHERIT | NP_WRITEBEHIND
#define  PIPE_MODE       NP_WAIT | NP_READMODE_MESSAGE | NP_TYPE_MESSAGE | 1
#define  PIPE_TIMEOUT    1000    // milliseconds till pipe times out
#define  OPEN_MODE       OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE | OPEN_FLAGS_WRITE_THROUGH
#define  SC_PIPE_MODE    OPEN_ACCESS_READWRITE    |  \
                         OPEN_SHARE_DENYREADWRITE |  \
                         OPEN_FLAGS_FAIL_ON_ERROR
#define  SC_PIPE_OPEN    OPEN_ACTION_OPEN_IF_EXISTS

#define  MAXUNAMES       20      /* maximum number of user names */

#define  MAX_TIMEOUT     10000

/*
 * Flags to logmsg().
 */
#define  IGN_CONS        0x001   /* don't print on console - only printsys( ) */
#define  SYNC_FILE       0x002   /* do fsync on file after printing - not used */
#define  ADDDATE         0x004   /* add a date to the message */
#define  MARK            0x008   /* this message is a mark */
#define  ADDHOST         0x016

#define  IPC_NAME      "\\socket\\syslog"
#define  SHMEM_NAME    "\\SHAREMEM\\SYSLOGD"
#define  MUX_NAME      "\\SEM32\\SYSLOGD\\MEMMUX"

#define _PATH_LOG      "\\socket\\syslog"
#define _PATH_LOGPID   getos2path("ETC", "syslog.pid")
#define _PATH_LOGCONF  getos2path("ETC", "syslog.conf")
#define _PATH_CONSOLE  "con"

#define  BUFFER_SLOTS  50
#define  BUFFER_SIZE   1024

typedef struct {
    int   index;                               // next free slot
    int   slotstat[BUFFER_SLOTS];              // 1- message in slot
    char  data[BUFFER_SLOTS - 1][BUFFER_SIZE]; // Buffer area messages are alloced from.
} SHAREMEM;

#define SMEM_SIZE sizeof(SHAREMEM)

/* values for f_type */
#define F_UNUSED        0               /* unused entry */
#define F_FILE          1               /* regular file */
#define F_CONSOLE       2                /* console terminal */
#define F_FORW          3               /* remote machine */


#include  <ctime>            // includes needed here to define
#include  <netdb>            // struct filed
#include  <netinet/in.h>

using std::time_t;

#include  "syslog.h"


struct STATS {
    unsigned  int  ipcoverflows;
    unsigned  int  tcpoverflows;
    unsigned  int  snmpoverflows;
    unsigned  int  sysipcrecvmsg;
    unsigned  int  systcprecvmsg;
    unsigned  int  syssnmprecvmsg;
    unsigned  int  sysintrecvmsg;
    unsigned  int  sysdmprecvmsg;
    unsigned  int  bsdprocessmsg;
    unsigned  int  markmsgs;
    unsigned  int  malformed;
};

struct PROGRAMOPTS {
    unsigned  int   DEBUG;
    unsigned  int   klog;
    unsigned  int   norepeat;
    unsigned  int   initialized;
    unsigned  long  marktimer;
    unsigned  long  logpriority;
    unsigned  int   securemode;
    unsigned  int   forwardmode;
    char  *cfgfilename;
    char  *cfgfilefull;
    char  popupfile[20];
    char  ft2exfile[20];
    char  altbindaddr[20];
    bool  threadipc;    // flag to indicate IPC thread start/running
    bool  threadtcp;    // flag to indicate TCP thread start/running
    bool  threadsnmp;
};


struct CURRENTMSG {
    int  pri;                        // PRIORITY - FACILITY and SEVERITY
    char timestamp[16];              // HEADER   - TIMESTAMP
    char hostname[MAXHOSTNAMELEN+1]; // HEADER   - HOSTNAME
    char msg[1024];                  // MSG      - TAG and CONTENT
    char orgmsg[1024];               // Holds original message
    int  orglen;                     // Holds original msg length
};


/*
 * This structure represents the files that will have log
 * copies printed.
 */
struct filed {
    struct  filed *f_next;                    // next in linked list
    short   f_type;                           // entry type, see below
    short   f_file;                           // file descriptor
    time_t  f_time;                           // time this was last written
    unsigned int f_pmask[LOG_NFACILITIES+1];  // priority mask

    union
    {
        struct
        {
            char    f_hname[MAXHOSTNAMELEN+1];
            struct  sockaddr_in  f_addr;
        } f_forw;                             // forwarding address

        char  f_fname[_MAX_PATH];             // filename for logging

    } f_un;

    char    f_prevline[1024];                 // last message logged
    time_t  f_lasttime;                       // time of last occurrence
    char    f_prevhost[MAXHOSTNAMELEN+1];     // host from which recd.
    int     f_prevpri;                        // pri of f_prevline
    int     f_prevlen;                        // length of f_prevline
    int     f_prevcount;                      // repetition cnt of prevline
    int     f_repeatcount;                    // number of "repeated" msgs

}; //$* end of filed stucture

#endif

