/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0       Michael K Greene <greenemk@cox.net>
 *                      September 2004
 *
 *                      Based on original BSD Syslogd
 *
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  The syslogd SNMP thread module.  Started with SNMP option
 *               received on Ports 161 and 162.
 *
 ***************************************************************************/

#define  INCL_DOSPROCESS
#define  INCL_DOSSEMAPHORES


#include  <os2.h>
#include  <iostream>
#include  <types>
#include  <sys\socket>
#include  <netinet\in>
#include  <unistd>
#include  <process.h>
#include  "syslogd.hpp"

using std::cout;
using std::endl;
using std::strlen;
using std::strcpy;
using std::strncat;
using std::memset;

void syslogdsnmp( PVOID );                           // main start of thread

static int  setupsnmp( void );                       // set up UDP sockets
static void closesnmp( void );                       // close up and end
extern void handleinternalmsg(int msgnum, int sev);  // handle errors

// Shared memory buffer defined in syslogd.cpp
extern SHAREMEM *pShareMem;
extern PSZ      szShareMem;
extern PSZ      MEMMUX;           // MUX to signal
extern HMTX     mtxMEMMUX;

// SEMAPHOREs define in syslogd main
extern PSZ   SNMPSEM1;       // SEMAPHORE to signal SNMP start / stop
extern HEV   hevSNMPSEM1;
extern ULONG SNMPSEM1Ct;

// SEMAPHOREs define in syslogd main
extern PSZ   SNMPSEM2;       // SEMAPHORE to signal
extern HEV   hevSNMPSEM2;
extern ULONG SNMPSEM2Ct;

extern  STATS  stats;

static int snmpsock1;               // socket for port 161
static int snmpsock2;               // socket for port 162


// syslogdsnmp( )
//
//
void syslogdsnmp( PVOID )
{
    bool  firststart = TRUE;
    int   msgrecv;
    char  recv_buffer[1024]  = {0};
    char  messagebuff[1024]  = {0};

    // setup for running, if error bug out - don't worry about main
    // will timeout waiting for SNMPSEM1.
    int seterr = setupsnmp( );

    if( seterr )
    {
        if(seterr > 1 && seterr < 5)
        {
            psock_errno( NULL );
            closesnmp( );
        }
        if(seterr == 5) cout << "SYSLOGSNMP: DosGetSharedMem failed" << cout;
    }

    char pri_1[ ] = "<109>snmp: ";
    char pri_2[ ] = "<117>snmptrap: ";

    int  socks[2];                // array of sockets
    int  sock_ready;              // socket receiving 1 or 2

    // main working loop **********************
    while(1)
    {
        // Just my play.  Set SEM1 (SYSLOGD is waiting), delay 1 sec
        // (let SYSLOGD reset), and start now FALSE - continue -
        // sleep 1.5 sec for SEM sets or thread will shutdown
        if(firststart)
        {
            DosPostEventSem(hevSNMPSEM1); // tell main I am up!!
            DosSleep(1500);
            firststart = FALSE;
        }

        // Don't need this - killing cycles ! remove from other threads
        // clear the buffer
        strnset(recv_buffer, '\0', sizeof(recv_buffer));

        socks[0] = snmpsock1;
        socks[1] = snmpsock2;

        sock_ready = os2_select(socks, 2, 0, 0, (long)MAX_TIMEOUT);

        if(sock_ready == -1) cout << "SYSLOGSNMP: socket select error" << endl;

        if(sock_ready)
        {
            // which socket to read
            if(socks[0] != -1)
            {
                msgrecv = recv(snmpsock1, recv_buffer, 1024, 0);
                strcpy(messagebuff, pri_1);
                strncat(messagebuff, recv_buffer, (1024-strlen(pri_1)));
            }

            if(socks[1] != -1)
            {
                msgrecv = recv(snmpsock2, recv_buffer, 1024, 0);
                strcpy(messagebuff, pri_2);
                strncat(messagebuff, recv_buffer, (1024-strlen(pri_2)));
            }

            // cout << messagebuff << endl;

            // have a message msgrecv > 0
            if(msgrecv > 0)
            {
                if(DosRequestMutexSem(mtxMEMMUX, 60000L) != 0)
                {
                    cout << "SYSLOGSNMP: Timeout waiting for MUX" << endl;
                }
                else
                {
                    if(pShareMem->slotstat[pShareMem->index])
                    {
                        cout << "SYSLOGSNMP: Buffer full!!" << endl;
                        stats.snmpoverflows++;
                    }
                    else
                    {
                        // clear the slot
                        strnset(pShareMem->data[pShareMem->index], '\0 ',
                                sizeof(pShareMem->data[pShareMem->index]));

                        // write message into slot
                        // replace any character < 32 or > 126 with a space
                        // RFC 3164 characters must be in this range
                        for(int i = 0; i < strlen(messagebuff); i++)
                        {
                            /*
                            if(((int)messagebuff[i] < 32) || ((int)messagebuff[i] > 126))
                                 pShareMem->data[pShareMem->index][i] = ' ';
                            else pShareMem->data[pShareMem->index][i] = messagebuff[i];
                            */
                            pShareMem->data[pShareMem->index][i] = messagebuff[i];
                        }

                        // set flag slot has message
                        pShareMem->slotstat[pShareMem->index] = 1;

                        // increment index to next slot
                        ++pShareMem->index;
                        ++stats.syssnmprecvmsg;

                        // if reached top of buffer reset index to 0 or bottom
                        if(pShareMem->index == (BUFFER_SLOTS)) pShareMem->index = 0;
                    }

                    DosReleaseMutexSem(mtxMEMMUX);
                }
            }

        } // new end sock_ready

        // check for shutdown signal from main
        DosQueryEventSem(hevSNMPSEM2, &SNMPSEM2Ct);
        if(SNMPSEM2Ct && !firststart)
        {
            cout << "SYSLOGSNMP: Thread shutdown started" << endl;
            DosResetEventSem(hevSNMPSEM1, &SNMPSEM1Ct);
            closesnmp( );
        }

    }  //$* end working loop

}  //$* end of syslogdsnmp


// setupsnmp( )
//
//  Setup socket, bind, and get access to buffer
//
int setupsnmp( )
{
    int    retcode = 0;
    struct sockaddr_in  socket_1;    // port 161
    struct sockaddr_in  socket_2;    // port 162

    struct servent      *sp;

    sp = getservbyname("snmp", "udp");
    int  port_1 = sp->s_port;
    sp = getservbyname("snmptrap", "udp");
    int  port_2 = sp->s_port;

    // Setup socket - descriptor sock 1 and 2
    snmpsock1 = socket(PF_INET, SOCK_DGRAM, 0);
    if(snmpsock1 == -1) return(1);
    snmpsock2 = socket(PF_INET, SOCK_DGRAM, 0);
    if(snmpsock2 == -1) return(2);

    // clear sockaddr struct and set values
    memset(&socket_1, 0, sizeof(socket_1));
    socket_1.sin_len         = sizeof(socket_1);
    socket_1.sin_family      = AF_INET;
    socket_1.sin_addr.s_addr = htonl(INADDR_ANY); // inet_addr("192.168.1.2"); htonl(gethostid(  )); //
    socket_1.sin_port        = port_1;

    memset(&socket_2, 0, sizeof(socket_2));
    socket_2.sin_len         = sizeof(socket_2);
    socket_2.sin_family      = AF_INET;
    socket_2.sin_addr.s_addr = htonl(INADDR_ANY); // inet_addr("192.168.1.2"); htonl(gethostid(  )); //
    socket_2.sin_port        = port_2;

    // bind to socket
    if(bind(snmpsock1, (struct sockaddr *)&socket_1, sizeof(socket_1))) return(3);
    if(bind(snmpsock2, (struct sockaddr *)&socket_2, sizeof(socket_2))) return(4);

    // get access to the buffer
    if(DosGetNamedSharedMem((PPVOID)&pShareMem,
                            SHMEM_NAME,
                            PAG_READ | PAG_WRITE) != 0) return(5);

    return 0;

}  //$* end of setupsnmp


// closesnmp( )
//
//  Close the sockets, release buffer, set SNMPSEM2, and end thread
//
void closesnmp( )
{
    soclose(snmpsock1);
    soclose(snmpsock2);
    DosFreeMem(pShareMem);         // free the buffer memory
    DosPostEventSem(hevSNMPSEM1);  // signal syslogd main thread shutdown
    _endthread();

}  //$* end of setupsnmp


