/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0a      Michael K Greene <greenemk@cox.net>
 *                      January 2005
 *
 *
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  The syslogd pipe thread module.  Handles syslog control
 *               messages received on named pipe from syslogctrl.
 *
 ***************************************************************************/

#define  INCL_DOSPROCESS
#define  INCL_DOSNMPIPES
#define  INCL_DOSSEMAPHORES
#define  INCL_DOSERRORS

#define  INTERNAL_NOPRI   0x10    // the "no priority" priority mark "facility"
                                  // needed for conf dump and do not want SYSLOG_NAMES
#include  <os2.h>
#include  <iostream>
#include  <string.hpp>
#include  <stdio>
#include  <process.h>
#include  "syslogmsg.hpp"
#include  "syslogd.hpp"


void syslogdpipe( PVOID );          // start thread entry

static  void  shutdownpipe(void);
static  void  dumpstats( void );
static  void  dumpconf( void );
static  void  dumpreap( void );
extern  void  handleinternalmsg(int msgnum, int sev);

extern  struct  STATS  stats;       // struct holds messages received/dropped
extern  struct  filed  *HeadFiles;  // linked list containing logging filters

// SEMAPHOREs define in syslogd main
extern  PSZ    PIPESEM1;
extern  HEV    hevPIPESEM1;
extern  ULONG  PIPESEM1Ct;

extern  HEV    hevPIPESEM2;
extern  HEV    hevPIPESEM2;
extern  ULONG  PIPESEM2Ct;

extern  PSZ    PIPESEM3;
extern  HEV    hevPIPESEM3;
extern  ULONG  PIPESEM3Ct;

static  HPIPE  SyslogPipe;  // handle to pipe start requests are made through
static  ULONG  bytes_read;
static  ULONG  bytes_write;
static  char   pipe_buffer[81] = {0};
static  char   *TypeNames[4] = { "UNUSED", "FILE", "CONSOLE", "FORW" };

// syslogpipe( )
//
//  Start thread from cron main.  A named pipe is openned and if no error
//  start main loop to read and write AT commands / query.
//
void syslogdpipe( PVOID )
{
    // Open the pipe main will timeout if fails
    if(DosCreateNPipe(PIPE_NAME,
                      &SyslogPipe,
                      PIPE_OPEN_MODE,
                      PIPE_MODE,
                      sizeof(pipe_buffer)*3,
                      sizeof(pipe_buffer)*3,
                      PIPE_TIMEOUT))  _endthread();

    bool firststart = TRUE;   // Prevents pre-intialization wrap shutdown

    // Main working loop
    //
    while(TRUE)
    {

        // Just my play.  Set SEM1 (SYSLOGD is waiting), delay 1 sec
        // (let SYSLOGD reset), and start now FALSE - continue -
        // sleep 1.5 sec for SEM sets or thread will shutdown
        if(firststart)
        {
            DosPostEventSem(hevPIPESEM1); // tell main I am up!!
            DosSleep(1500);
            firststart = FALSE;
        }

        if(DosConnectNPipe(SyslogPipe)) handleinternalmsg(PIPEERROR_1, LOG_ERR);
        else
        {
            *pipe_buffer=0;

            if(DosRead(SyslogPipe,pipe_buffer, sizeof(pipe_buffer), &bytes_read))
            {
                DosDisConnectNPipe(SyslogPipe);
                handleinternalmsg(PIPEERROR_2, LOG_ERR);
            }

            // Receive stats query
            if(strstr(pipe_buffer, "PIPESTATSEND")) dumpstats( );

            // receive conf query
            if(strstr(pipe_buffer, "PIPECONFSEND")) dumpconf( );

            // receive conf query
            if(strstr(pipe_buffer, "PIPEREAPSEND")) dumpreap( );

            // receive shutdown from syslogctrl
            if(strstr(pipe_buffer, "PIPESHUTDOWN"))
            {
                DosPostEventSem(hevPIPESEM2);
                shutdownpipe( );
            }

            // Test signal to main
            if(strstr(pipe_buffer, "BINGO"))
            {
                cout << "BINGO" << endl;
                strcpy(pipe_buffer, "RECEIVED BING");
                DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write);

                // DosPostEventSem(hevPIPESEM3);
            }

        }

        if(DosDisConnectNPipe(SyslogPipe)) handleinternalmsg(PIPEERROR_3, LOG_ERR);

    }

}  //$* end of syslogpipe( )


// shutdowpipe()
//
//  This function is used by SYSLOGPIPE process is being
//  shut down.  We need to close the pipe.
//
void shutdownpipe(void)
{
    DosClose(SyslogPipe);
    DosPostEventSem(hevPIPESEM1); // signal syslogd main thread shutdown
    _endthread( );

}  //$* end of shutdown_cronat


void dumpstats( void )
{
    sprintf(pipe_buffer, "Thread messages received  - UDP: %d  IPC: %d  Total: %d",
                          stats.systcprecvmsg, stats.sysipcrecvmsg,
                          (stats.systcprecvmsg + stats.sysipcrecvmsg));
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    sprintf(pipe_buffer, "Messages received by main - %d", stats.sysdmprecvmsg);
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    sprintf(pipe_buffer, "Messages processed -        %d \n", stats.bsdprocessmsg);
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    sprintf(pipe_buffer, "Thread overflows -          UDP: %d  IPC: %d  Total: %d",
                          stats.tcpoverflows, stats.ipcoverflows,
                          (stats.tcpoverflows + stats.ipcoverflows));
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    sprintf(pipe_buffer, "Mark messages processed -   %d", stats.markmsgs);
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    strcpy(pipe_buffer, "DONE");
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    // Just wait here until syslogctrl finish and sends OK
    DosRead(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_read);

}  //$* end of statdump


// dumpconf ( )
//
//  Dumps the current program logging configuration loaded from syslog.conf
//  original routine was located in init( ) Removed to an independant function
//  so can query via named pipe
//
void dumpconf( void )
{
    struct filed *f;
    char   tmp_buffer[2];
    char   msg_buffer[81] = {0};

    for (f = HeadFiles; f; f = f->f_next)
    {
        for (int i = 0; i <= LOG_NFACILITIES; i++)
                         if (f->f_pmask[i] == INTERNAL_NOPRI) strcat(msg_buffer, "X ");
                         else
                         {
                             _itoa(f->f_pmask[i], tmp_buffer, 10);
                             strcat(msg_buffer, tmp_buffer);
                             strcat(msg_buffer, " ");
                         }

        strcat(msg_buffer, TypeNames[f->f_type]);
        strcat(msg_buffer, ": ");

        switch (f->f_type)
        {
            case F_FILE:
            case F_CONSOLE: strcat(msg_buffer, f->f_un.f_fname);
                            break;
            case F_FORW:    strcat(msg_buffer, f->f_un.f_forw.f_hname);
                            break;
        }

        strcpy(pipe_buffer, msg_buffer);

        *msg_buffer = 0;

        if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);
    }

    strcpy(pipe_buffer, "DONE");
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    // Just wait here until syslogctrl finish and sends OK
    DosRead(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_read);

}  //$* end of confdump




void dumpreap( void )
{
    struct filed *f;
    char   msg_buffer[81] = {0};

// repeatinterval array of 3 int, size of int = 4 then (12/4)-1 = 2
// MAXREPEAT will always be 2 unless int size < 4 or add times
int     repeatinterval[] = { 30, 120, 600 };    /* # of secs before flush */
#define MAXREPEAT     ((sizeof(repeatinterval) / sizeof(repeatinterval[0])) - 1)

// f_time + repeatinterval[ ] (30 120 600)
#define REPEATTIME(f) ((f)->f_time + repeatinterval[(f)->f_repeatcount])

    for (f = HeadFiles; f; f = f->f_next)
    {

// cout << f->f_time << endl;

cout << f->f_repeatcount << endl;


// strcpy(msg_buffer, f->f_time);
strcat(msg_buffer, " ");


        switch (f->f_type)
        {
            case F_FILE:
            case F_CONSOLE: strcat(msg_buffer, f->f_un.f_fname);
                            break;
            case F_FORW:    strcat(msg_buffer, f->f_un.f_forw.f_hname);
                            break;
        }

        strcpy(pipe_buffer, msg_buffer);

        *msg_buffer = 0;

        if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);
    }

    strcpy(pipe_buffer, "DONE");
    if(DosWrite(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_write))
                                             handleinternalmsg(PIPEERROR_4, LOG_ERR);

    // Just wait here until syslogctrl finish and sends OK
    DosRead(SyslogPipe, pipe_buffer, sizeof(pipe_buffer), &bytes_read);

}  //$* end of confdump

