/****************************************************************************
 *
 *           S Y S L O G D / 2    A UN*X SYSLOGD clone for OS/2
 *
 *  ========================================================================
 *
 *    Version 1.0       Michael K Greene <greenemk@cox.net>
 *                      October 2004
 *
 *                      Based on original BSD Syslogd
 *
 *
 *            SYSLOGD/2 compiles with OpenWatom (www.openwatcom.org).
 *
 *  ========================================================================
 *
 * Description:  If SUPPRESSPOPUPS=<drive> is set in config.sys will take
 *               popuplog.os2 and log as a kernel message via printsys( ).
 *               Requires -k option to be set.
 *
 ***************************************************************************/

#define INCL_DOSMISC
#define INCL_DOSERRORS
#define INCL_DOSPROCESS

#include <os2.h>
#include <iostream>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <ctype.h>
#include <fstream>
#include "syslogd.hpp"

using namespace std;
/* using std::cout;
using std::endl;
using std::ifstream;
using std::sprintf; */

extern PROGRAMOPTS prgoptions;
static char buffer[120];


void poplog( void );
static void getpopcfg( void );
extern void printsys(char *msg);

// poplog( )
//
//  Open the popuplog and sent to printsys( )
//
void poplog( )
{
    if(*prgoptions.popupfile == 0)
    {
        getpopcfg( );
        if(*prgoptions.popupfile == 0)
        {
            --prgoptions.klog;
            cout << "KLOG Disabled" << endl;
        }
    }

    int start = 1;

    ifstream POPLOG;
    POPLOG.open(prgoptions.popupfile);
    if( POPLOG.good( ) )
    {
        while( POPLOG.good( ) )
        {

            POPLOG.getline(buffer, 120, '\n');

            if(start == 1)
            {
                printsys(" --< START POPUPLOG ENTRY >-- ");
                start = 0;
            }

            if((int)buffer[0] == 45)
            {
                printsys(" --<  END POPUPLOG ENTRY  >-- ");
                start = 1;

                // wait 1 sec after each entry, if forwarding a large
                // popup log will fill receiving system buffer
                DosSleep(1000L);
            }
            else if((int)buffer[0] != 0) printsys( buffer );

        }

        printsys(" --<  END POPUPLOG ENTRY  >-- ");
        POPLOG.close( );
        remove(prgoptions.popupfile);
    }

    // just use the ifstream POPLOG for ft2ex file
    POPLOG.open(prgoptions.ft2exfile);

    if( POPLOG.good( ) )
    {
        while( POPLOG.good( ) )
        {

            POPLOG.getline(buffer, 120, '\n');
            printsys( buffer );
            DosSleep(1000L);
        }

        POPLOG.close( );
        remove(prgoptions.ft2exfile);
    }
}


// getpopcfg( )
//
//  Scan config.sys and setup the path to popuplog.os2
//
void getpopcfg( void )
{
    char   bootcfg[15];
    char   popfile[15];
    char   ft2file[15];

    ULONG  aulSysInfo[QSV_MAX] = {0};

    DosQuerySysInfo(1L, QSV_MAX, (PVOID)aulSysInfo, sizeof(ULONG)*QSV_MAX);
    sprintf(bootcfg, "%c%s",
            (char)(aulSysInfo[QSV_BOOT_DRIVE -1]+64), ":\\CONFIG.SYS");

    sprintf(ft2file, "%c%s", (char)(aulSysInfo[QSV_BOOT_DRIVE -1]+64), ":\\ft2excpt.log");

    strcpy(prgoptions.ft2exfile, ft2file);

    ifstream CONFIGSYS;

    CONFIGSYS.open(bootcfg);

    while( CONFIGSYS.good( ) )
    {
        CONFIGSYS.getline(buffer, 256, '\n');

        if(strstr(buffer, "SUPPRESSPOPUPS"))
        {
            int i;
            for(i = 0; i < strlen(buffer); i++) if(buffer[i] == '=') break;

            sprintf(popfile, "%c%s", buffer[++i], ":\\POPUPLOG.OS2");
            strcpy(prgoptions.popupfile, popfile);
        }
    }
    CONFIGSYS.close( );
}


